import numpy as np
import warnings
import os
from pyiron.gpaw.pyiron_ase import AseJob
from pyiron.atomistics.structure.atoms import pyiron_to_ase
from pyiron.base.generic.parameters import GenericParameters
from pyiron.base.settings.generic import Settings
from ase import Atoms

s = Settings()
os.environ["GPAW_SETUP_PATH"] = os.path.join(s.resource_paths[0], "gpaw", "potentials")

try:
    from gpaw import GPAW, PW, MethfesselPaxton
except ImportError:
    pass


class GpawJob(AseJob):
    def __init__(self, project, job_name):
        super(GpawJob, self).__init__(project, job_name)
        self.__name__ = "GpawJob"
        self.input = GpawInput()

    @property
    def structure(self):
        return self._structure

    @structure.setter
    def structure(self, basis):
        if not isinstance(basis, Atoms):
            basis = pyiron_to_ase(basis)
        self._structure = basis

    @property
    def encut(self):
        return self.plane_wave_cutoff

    @encut.setter
    def encut(self, val):
        self.plane_wave_cutoff = val

    @property
    def plane_wave_cutoff(self):
        return self.input["encut"]

    @plane_wave_cutoff.setter
    def plane_wave_cutoff(self, val):
        self.input["encut"] = val

    def get_k_mesh_by_cell(self, cell=None, kpoints_per_angstrom=1):
        if cell is None:
            cell = self.structure.cell
        latlens = [np.linalg.norm(lat) for lat in cell]
        kmesh = np.rint(
            np.array([2 * np.pi / ll for ll in latlens]) * kpoints_per_angstrom
        )
        if kmesh.min() <= 0:
            warnings.warn("kpoint per angstrom too low")
        return [int(k) for k in kmesh]

    def set_kpoints(
        self,
        mesh=None,
        scheme="MP",
        center_shift=None,
        symmetry_reduction=True,
        manual_kpoints=None,
        weights=None,
        reciprocal=True,
        kpoints_per_angstrom=None,
    ):
        """
        Function to setup the k-points

        Args:
            mesh (list): Size of the mesh (ignored if scheme is not set to 'MP' or kpoints_per_angstrom is set)
            scheme (str): Type of k-point generation scheme (MP/GP(gamma point)/Manual/Line)
            center_shift (list): Shifts the center of the mesh from the gamma point by the given vector in relative
                                 coordinates
            symmetry_reduction (boolean): Tells if the symmetry reduction is to be applied to the k-points
            manual_kpoints (list/numpy.ndarray): Manual list of k-points
            weights(list/numpy.ndarray): Manually supplied weights to each k-point in case of the manual mode
            reciprocal (bool): Tells if the supplied values are in reciprocal (direct) or cartesian coordinates (in
            reciprocal space)
            kpoints_per_angstrom (float): Number of kpoint per angstrom in each direction
        """
        if kpoints_per_angstrom is not None:
            if mesh is not None:
                warnings.warn("mesh value is overwritten by kpoints_per_angstrom")
            mesh = self.get_k_mesh_by_cell(kpoints_per_angstrom=kpoints_per_angstrom)
        if mesh is not None:
            if np.min(mesh) <= 0:
                raise ValueError("mesh values must be larger than 0")
        if center_shift is not None:
            if np.min(center_shift) < 0 or np.max(center_shift) > 1:
                warnings.warn("center_shift is given in relative coordinates")
        self._set_kpoints(
            mesh=mesh,
            scheme=scheme,
            center_shift=center_shift,
            symmetry_reduction=symmetry_reduction,
            manual_kpoints=manual_kpoints,
            weights=weights,
            reciprocal=reciprocal,
        )

    # Backward compatibility
    def get_encut(self):
        return self.encut

    def set_encut(self, encut):
        """
        Sets the plane-wave energy cutoff
        Args:
            encut (float): The energy cutoff in eV
        """
        self.plane_wave_cutoff = encut

    def _set_kpoints(
        self,
        mesh=None,
        scheme="MP",
        center_shift=None,
        symmetry_reduction=True,
        manual_kpoints=None,
        weights=None,
        reciprocal=True,
    ):
        if scheme != "MP":
            raise ValueError("Currently only MP is supported in the pyiron wrapper.")
        if center_shift is not None:
            raise ValueError("centershift is not implemented in the pyiron wrapper.")
        if not symmetry_reduction:
            raise ValueError(
                "symmetry_reduction is not implemented in the pyiron wrapper."
            )
        if manual_kpoints is not None:
            raise ValueError(
                "manual_kpoints are not implemented in the pyiron wrapper."
            )
        if weights is not None:
            raise ValueError("weights are not implemented in the pyiron wrapper.")
        if not reciprocal:
            raise ValueError("reciprocal is not implemented in the pyiron wrapper.")
        self.input["kpoints"] = mesh

    def write_input(self):
        pass

    def collect_output(self):
        pass

    def run_static(self):
        pre_run_mode = self.server.run_mode
        self.server.run_mode.interactive = True
        self.run_if_interactive()
        self.interactive_close()
        self.server.run_mode = pre_run_mode

    def run_if_scheduler(self):
        self._create_working_directory()
        super(GpawJob, self).run_if_scheduler()

    def run_if_interactive(self):
        if self.structure.calc is None:
            kpoints = self.input["kpoints"]
            if isinstance(kpoints, str):
                kpoints = (
                    self.input["kpoints"].replace("[", "").replace("]", "").split()
                )
            self._create_working_directory()
            calc = GPAW(
                mode=PW(float(self.input["encut"])),
                xc=self.input["potential"],
                occupations=MethfesselPaxton(width=float(self.input["sigma"])),
                kpts=kpoints,
                txt=self.working_directory + "/" + self.job_name + ".txt",
            )
            self.structure.set_calculator(calc)
        self.status.running = True
        self.structure.calc.calculate(self.structure)
        self.interactive_collect()

    def to_hdf(self, hdf=None, group_name=None):
        """
        Store the ExampleJob object in the HDF5 File

        Args:
            hdf (ProjectHDFio): HDF5 group object - optional
            group_name (str): HDF5 subgroup name - optional
        """
        super(GpawJob, self).to_hdf(hdf=hdf, group_name=group_name)
        with self.project_hdf5.open("input") as hdf5_input:
            self.input.to_hdf(hdf5_input)

    def from_hdf(self, hdf=None, group_name=None):
        """
        Restore the ExampleJob object in the HDF5 File

        Args:
            hdf (ProjectHDFio): HDF5 group object - optional
            group_name (str): HDF5 subgroup name - optional
        """
        super(GpawJob, self).from_hdf(hdf=hdf, group_name=group_name)
        with self.project_hdf5.open("input") as hdf5_input:
            self.input.from_hdf(hdf5_input)


class GpawInput(GenericParameters):
    """
    Input class for the ExampleJob based on the GenericParameters class.

    Args:
        input_file_name (str): Name of the input file - optional
    """

    def __init__(self, input_file_name=None):
        super(GpawInput, self).__init__(
            input_file_name=input_file_name, table_name="input", comment_char="#"
        )

    def load_default(self):
        """
        Loading the default settings for the input file.
        """
        input_str = """\
kpoints [6,6,6]
sigma 0.1
encut 350
potential PBE
"""
        self.load_string(input_str)



import sys
import getopt
from pyiron.base.job.wrapper import job_wrapper_function


def command_line(argv):
    """
    Parse the command line arguments.

    Args:
        argv: Command line arguments

    """
    debug = False
    project_path = None
    job_id = None
    file_path = None
    submit_on_remote = False
    try:
        opts, args = getopt.getopt(
            argv, "dj:p:f:sh", ["debug", "project_path=", "file_path=", "job_id=", "submit", "help"]
        )
    except getopt.GetoptError:
        print("cms.py --p <project_path> -j <job_id> <debug>")
        sys.exit()
    else:
        for opt, arg in opts:
            if opt in ("-h", "--help"):
                print("cms.py --p <project_path> -j <job_id> <debug>")
                sys.exit()
            elif opt in ("-d", "--debug"):
                debug = True
            elif opt in ("-j", "--job_id"):
                job_id = arg
            elif opt in ("-p", "--project_path"):
                project_path = arg
            elif opt in ("-f", "--file_path"):
                file_path = arg
            elif opt in ("-s", "--submit"):
                submit_on_remote = True
        job_wrapper_function(
            working_directory=project_path,
            job_id=job_id,
            file_path=file_path,
            debug=debug,
            submit_on_remote=submit_on_remote
        )
        sys.exit()


if __name__ == "__main__":
    command_line(sys.argv[1:])



# coding: utf-8
# Copyright (c) Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department
# Distributed under the terms of "New BSD License", see the LICENSE file.

import pandas
import sys
import os
import numpy as np
from pyiron.base.settings.generic import Settings

"""
The Jobtable module provides a set of top level functions to interact with the database.
"""

__author__ = "Jan Janssen"
__copyright__ = (
    "Copyright 2019, Max-Planck-Institut für Eisenforschung GmbH - "
    "Computational Materials Design (CM) Department"
)
__version__ = "1.0"
__maintainer__ = "Jan Janssen"
__email__ = "janssen@mpie.de"
__status__ = "production"
__date__ = "Sep 1, 2017"

s = Settings()


def _job_dict(
    database,
    sql_query,
    user,
    project_path,
    recursive,
    job=None,
    sub_job_name="%",
    element_lst=None,
):
    """
    Internal function to access the database from the project directly.

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        recursive (bool): search subprojects [True/False]
        job (str): job_name - by default None
        sub_job_name (str): path inside the HDF5 file - "%" by default to accept any path
        element_lst (list): list of elements required in the chemical formular - by default None

    Returns:
        list: the function returns a list of dicts like get_items_sql, but it does not format datetime:
             [{'chemicalformula': u'Ni108',
              'computer': u'mapc157',
              'hamilton': u'LAMMPS',
              'hamversion': u'1.1',
              'id': 24,
              'job': u'DOF_1_0',
              'parentid': 21L,
              'project': u'lammps.phonons.Ni_fcc',
              'projectpath': u'D:/PyIron/PyIron_data/projects',
              'status': u'finished',
              'timestart': datetime.datetime(2016, 6, 24, 10, 17, 3, 140000),
              'timestop': datetime.datetime(2016, 6, 24, 10, 17, 3, 173000),
              'totalcputime': 0.033,
              'username': u'test'},
             {'chemicalformula': u'Ni108',
              'computer': u'mapc157',
              'hamilton': u'LAMMPS',
              'hamversion': u'1.1',
              'id': 21,
              'job': u'ref',
              'parentid': 20L,
              'project': u'lammps.phonons.Ni_fcc',
              'projectpath': u'D:/PyIron/PyIron_data/projects',
              'status': u'finished',
              'timestart': datetime.datetime(2016, 6, 24, 10, 17, 2, 429000),
              'timestop': datetime.datetime(2016, 6, 24, 10, 17, 2, 463000),
              'totalcputime': 0.034,
              'username': u'test'},.......]

    """
    dict_clause = {}
    # FOR GET_ITEMS_SQL: clause = []
    if user is not None:
        dict_clause["username"] = str(user)
        # FOR GET_ITEMS_SQL: clause.append("username = '" + self.user + "'")
    if sql_query is not None:
        # FOR GET_ITEMS_SQL: clause.append(self.sql_query)
        if "AND" in sql_query:
            cl_split = sql_query.split(" AND ")
        elif "and" in sql_query:
            cl_split = sql_query.split(" and ")
        else:
            cl_split = [sql_query]
        dict_clause.update(
            {str(element.split()[0]): element.split()[2] for element in cl_split}
        )
    if job is not None:
        dict_clause["job"] = str(job)

    if recursive:
        dict_clause["project"] = str(project_path) + "%"
    else:
        dict_clause["project"] = str(project_path)
    if sub_job_name is None:
        dict_clause["subjob"] = None
    elif sub_job_name != "%":
        dict_clause["subjob"] = str(sub_job_name)
    if element_lst is not None:
        dict_clause["element_lst"] = element_lst

    s.logger.debug("sql_query: %s", str(dict_clause))
    return database.get_items_dict(dict_clause)


def get_db_columns(database):
    """
    Get column names

    Args:
        database (DatabaseAccess): Database object

    Returns:
        list: list of column names like:
             ['id',
             'parentid',
             'masterid',
             'projectpath',
             'project',
             'job',
             'subjob',
             'chemicalformula',
             'status',
             'hamilton',
             'hamversion',
             'username',
             'computer',
             'timestart',
             'timestop',
             'totalcputime']
    """
    return database.get_table_headings()


def job_table(
    database,
    sql_query,
    user,
    project_path,
    recursive=True,
    columns=None,
    all_columns=False,
    sort_by="id",
    max_colwidth=200,
    element_lst=None,
    job_name_contains='',
):
    """
    Access the job_table

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        recursive (bool): search subprojects [True/False]
        columns (list): by default only the columns ['job', 'project', 'chemicalformula'] are selected, but the
                        user can select a subset of ['id', 'status', 'chemicalformula', 'job', 'subjob', 'project',
                        'projectpath', 'timestart', 'timestop', 'totalcputime', 'computer', 'hamilton', 'hamversion',
                        'parentid', 'masterid']
        all_columns (bool): Select all columns - this overwrites the columns option.
        sort_by (str): Sort by a specific column
        max_colwidth (int): set the column width
        element_lst (list): list of elements required in the chemical formular - by default None
        job_name_contains (str): a string which should be contained in every job_name

    Returns:
        pandas.Dataframe: Return the result as a pandas.Dataframe object
    """
    if columns is None:
        columns = ["job", "project", "chemicalformula"]
    all_db = [
        "id",
        "status",
        "chemicalformula",
        "job",
        "subjob",
        "projectpath",
        "project",
        "timestart",
        "timestop",
        "totalcputime",
        "computer",
        "hamilton",
        "hamversion",
        "parentid",
        "masterid",
    ]

    if all_columns:
        columns = all_db
    job_dict = _job_dict(
        database=database,
        sql_query=sql_query,
        user=user,
        project_path=project_path,
        recursive=recursive,
        element_lst=element_lst,
    )
    pandas.set_option("display.max_colwidth", max_colwidth)
    df = pandas.DataFrame(job_dict)
    if len(job_dict) == 0:
        return df
    if job_name_contains != '':
        df = df[df.job.str.contains(job_name_contains)]
    if sort_by in columns:
        return df[columns].sort_values(by=sort_by)
    return df[columns]


def get_jobs(database, sql_query, user, project_path, recursive=True, columns=None):
    """
    Internal function to return the jobs as dictionary rather than a pandas.Dataframe

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        recursive (bool): search subprojects [True/False]
        columns (list): by default only the columns ['id', 'project'] are selected, but the user can select a subset
                        of ['id', 'status', 'chemicalformula', 'job', 'subjob', 'project', 'projectpath', 'timestart',
                        'timestop', 'totalcputime', 'computer', 'hamilton', 'hamversion', 'parentid', 'masterid']

    Returns:
        dict: columns are used as keys and point to a list of the corresponding values
    """
    if columns is None:
        columns = ["id", "project"]
    df = job_table(database, sql_query, user, project_path, recursive, columns=columns)
    if len(df) == 0:
        dictionary = {}
        for key in columns:
            dictionary[key] = list()
        return dictionary
        # return {key: list() for key in columns}
    dictionary = {}
    for key in df.keys():
        dictionary[key] = df[
            key
        ].tolist()  # ToDo: Check difference of tolist and to_list
    return dictionary
    # return {key: df[key].tolist() for key in df.keys()}


def get_job_ids(database, sql_query, user, project_path, recursive=True):
    """
    Return the job IDs matching a specific query

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        recursive (bool): search subprojects [True/False]

    Returns:
        list: a list of job IDs
    """
    return get_jobs(database, sql_query, user, project_path, recursive=recursive)["id"]


def get_child_ids(database, sql_query, user, project_path, job_specifier, status=None):
    """
    Get the childs for a specific job

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        job_specifier (str): name of the master job or the master jobs job ID
        status (str): filter childs which match a specific status - None by default

    Returns:
        list: list of child IDs
    """
    id_master = get_job_id(database, sql_query, user, project_path, job_specifier)
    if id_master is None:
        return []
    else:
        search_dict = {"masterid": str(id_master)}
        if status is not None:
            search_dict["status"] = status
        return sorted(
            [
                job["id"]
                for job in database.get_items_dict(
                    search_dict, return_all_columns=False
                )
            ]
        )


def get_job_id(database, sql_query, user, project_path, job_specifier):
    """
    get the job_id for job named job_name in the local project path from database

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        job_specifier (str): name of the job or job ID

    Returns:
        int: job ID of the job
    """
    if sys.version_info.major == 2:
        if isinstance(job_specifier, (int, long, np.integer)):
            return int(job_specifier)  # is id
    else:
        if isinstance(job_specifier, (int, np.integer)):
            return job_specifier  # is id

    job_specifier.replace(".", "_")
    # if job_specifier[0] is not '/':
    #     sub_job_name = '/' + job_specifier
    # else:
    #     sub_job_name = job_specifier
    # job_dict = _job_dict(database, sql_query, user, project_path, recursive=False,  # job=job_specifier,
    #                      sub_job_name=sub_job_name)
    # if len(job_dict) == 0:
    #     job_dict = _job_dict(database, sql_query, user, project_path, recursive=True,  # job=job_specifier,
    #                          sub_job_name=sub_job_name)
    job_dict = _job_dict(
        database, sql_query, user, project_path, recursive=False, job=job_specifier
    )
    if len(job_dict) == 0:
        job_dict = _job_dict(
            database, sql_query, user, project_path, recursive=True, job=job_specifier
        )
    if len(job_dict) == 0:
        return None
    elif len(job_dict) == 1:
        return job_dict[0]["id"]
    else:
        raise ValueError(
            "job name '{0}' in this project is not unique".format(job_dict)
        )


def set_job_status(database, sql_query, user, project_path, job_specifier, status):
    """
    Set the status of a particular job

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        job_specifier (str): name of the job or job ID
        status (str): job status can be one of the following ['initialized', 'appended', 'created', 'submitted',
                     'running', 'aborted', 'collect', 'suspended', 'refresh', 'busy', 'finished']

    """
    database.item_update(
        {"status": str(status)},
        get_job_id(database, sql_query, user, project_path, job_specifier),
    )


def get_job_status(database, sql_query, user, project_path, job_specifier):
    """
    Get the status of a particular job

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        job_specifier (str): name of the job or job ID

    Returns:
        str: job status can be one of the following ['initialized', 'appended', 'created', 'submitted', 'running',
             'aborted', 'collect', 'suspended', 'refresh', 'busy', 'finished']
    """
    try:
        return database.get_item_by_id(
            get_job_id(database, sql_query, user, project_path, job_specifier)
        )["status"]
    except KeyError:
        return None


def get_job_working_directory(database, sql_query, user, project_path, job_specifier):
    """
    Get the working directory of a particular job

    Args:
        database (DatabaseAccess): Database object
        sql_query (str): SQL query to enter a more specific request
        user (str): username of the user whoes user space should be searched
        project_path (str): root_path - this is in contrast to the project_path in GenericPath
        job_specifier (str): name of the job or job ID

    Returns:
        str: working directory as absolute path
    """
    try:
        db_entry = database.get_item_by_id(
            get_job_id(database, sql_query, user, project_path, job_specifier)
        )
        if db_entry:
            job_name = db_entry["subjob"][1:]
            return os.path.join(
                db_entry["projectpath"],
                db_entry["project"],
                job_name + "_hdf5",
                job_name,
            )
        else:
            return None
    except KeyError:
        return None



# coding: utf-8
# Copyright (c) Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department
# Distributed under the terms of "New BSD License", see the LICENSE file.

from __future__ import print_function
import numpy as np
import os
import posixpath
from pyiron.base.generic.parameters import GenericParameters
from pyiron.base.job.generic import GenericJob
from pyiron.base.pyio.parser import Logstatus
from pyiron.atomistics.job.interactive import GenericInteractive

"""
Example Job class for testing the pyiron classes
"""

__author__ = "Joerg Neugebauer, Jan Janssen"
__copyright__ = (
    "Copyright 2019, Max-Planck-Institut für Eisenforschung GmbH - "
    "Computational Materials Design (CM) Department"
)
__version__ = "1.0"
__maintainer__ = "Jan Janssen"
__email__ = "janssen@mpie.de"
__status__ = "production"
__date__ = "Sep 1, 2017"


class ExampleJob(GenericJob):
    """
    ExampleJob generating a list of random numbers to simulate energy fluctuations.

    Args:
        project (ProjectHDFio): ProjectHDFio instance which points to the HDF5 file the job is stored in
        job_name (str): name of the job, which has to be unique within the project

    Attributes:

        .. attribute:: job_name

            name of the job, which has to be unique within the project

        .. attribute:: status

            execution status of the job, can be one of the following [initialized, appended, created, submitted, running,
                                                                      aborted, collect, suspended, refresh, busy, finished]

        .. attribute:: job_id

            unique id to identify the job in the pyiron database

        .. attribute:: parent_id

            job id of the predecessor job - the job which was executed before the current one in the current job series

        .. attribute:: master_id

            job id of the master job - a meta job which groups a series of jobs, which are executed either in parallel or in
            serial.

        .. attribute:: child_ids

            list of child job ids - only meta jobs have child jobs - jobs which list the meta job as their master

        .. attribute:: project

            Project instance the jobs is located in

        .. attribute:: project_hdf5

            ProjectHDFio instance which points to the HDF5 file the job is stored in

        .. attribute:: job_info_str

            short string to describe the job by it is job_name and job ID - mainly used for logging

        .. attribute:: working_directory

            working directory of the job is executed in - outside the HDF5 file

        .. attribute:: path

            path to the job as a combination of absolute file system path and path within the HDF5 file.

        .. attribute:: version

            Version of the hamiltonian, which is also the version of the executable unless a custom executable is used.

        .. attribute:: executable

            Executable used to run the job - usually the path to an external executable.

        .. attribute:: library_activated

            For job types which offer a Python library pyiron can use the python library instead of an external executable.

        .. attribute:: server

            Server object to handle the execution environment for the job.

        .. attribute:: queue_id

            the ID returned from the queuing system - it is most likely not the same as the job ID.

        .. attribute:: logger

            logger object to monitor the external execution and internal pyiron warnings.

        .. attribute:: restart_file_list

            list of files which are used to restart the calculation from these files.

        .. attribute:: job_type

            Job type object with all the available job types: ['ExampleJob', 'SerialMaster', 'ParallelMaster', 'ScriptJob',
                                                               'ListMaster']
    """

    def __init__(self, project, job_name):
        super(ExampleJob, self).__init__(project, job_name)
        self.__version__ = "0.3"
        self.__name__ = "ExampleJob"
        self.input = ExampleInput()
        self.executable = "python -m pyiron.testing.executable"
        self._interactive_cache = {"alat": [], "count": [], "energy": []}

    def set_input_to_read_only(self):
        """
        This function enforces read-only mode for the input classes, but it has to be implement in the individual
        classes.
        """
        self.input.read_only = True

    # define routines that create all necessary input files
    def write_input(self):
        """
        Call routines that generate the codespecifc input files
        """
        self.input.write_file(file_name="input.inp", cwd=self.working_directory)

    # define routines that collect all output files
    def collect_output(self):
        """
        Parse the output files of the example job and store the results in the HDF5 File.
        """
        self.collect_output_log()
        self.collect_warnings()
        self.collect_logfiles()

    def collect_output_log(self, file_name="output.log"):
        """
        general purpose routine to extract output from logfile

        Args:
            file_name (str): output.log - optional
        """
        tag_dict = {
            "alat": {"arg": "0", "rows": 0},
            "count": {"arg": "0", "rows": 0},
            "energy": {"arg": "0", "rows": 0},
        }
        lf = Logstatus()
        file_name = posixpath.join(self.working_directory, file_name)
        lf.extract_file(file_name=file_name, tag_dict=tag_dict)
        with self.project_hdf5.open("output/generic") as h5:
            lf.to_hdf(h5)
            h5["energy_tot"] = np.array(h5["energy"])
            h5["volume"] = np.array(h5["alat"])

    def collect_warnings(self):
        """
        Collect the warnings if any were written to the info.log file and store them in the HDF5 file
        """
        warnings_lst = []
        with open(posixpath.join(self.working_directory, "info.log"), "r") as f:
            lines = f.readlines()
        for line in lines:
            if "WARNING" in line:
                warnings_lst.append(line.split("WARNING"))
                warnings_lst[-1][-1] = warnings_lst[-1][-1].rstrip()
        if len(warnings_lst) > 0:
            warnings_dict = {
                "Module": [warnings_lst[i][0] for i in range(len(warnings_lst))],
                "Message": [warnings_lst[i][1] for i in range(len(warnings_lst))],
            }
            print("module: ", warnings_lst[:][:])
            with self.project_hdf5.open("output"):
                self._hdf5["WARNINGS"] = warnings_dict

    def collect_logfiles(self):
        """
        Collect the errors from the info.log file and store them in the HDF5 file
        """
        errors_lst = []
        with open(posixpath.join(self.working_directory, "info.log"), "r") as f:
            lines = f.readlines()
        for line in lines:
            if "ERROR" in line:
                errors_lst.append(line)
        if len(errors_lst) > 0:
            with self.project_hdf5.open("output") as hdf_output:
                hdf_output["ERRORS"] = errors_lst

    def to_hdf(self, hdf=None, group_name=None):
        """
        Store the ExampleJob object in the HDF5 File

        Args:
            hdf (ProjectHDFio): HDF5 group object - optional
            group_name (str): HDF5 subgroup name - optional
        """
        super(ExampleJob, self).to_hdf(hdf=hdf, group_name=group_name)
        with self.project_hdf5.open("input") as hdf5_input:
            self.input.to_hdf(hdf5_input)

    def from_hdf(self, hdf=None, group_name=None):
        """
        Restore the ExampleJob object in the HDF5 File

        Args:
            hdf (ProjectHDFio): HDF5 group object - optional
            group_name (str): HDF5 subgroup name - optional
        """
        super(ExampleJob, self).from_hdf(hdf=hdf, group_name=group_name)
        with self.project_hdf5.open("input") as hdf5_input:
            self.input.from_hdf(hdf5_input)

    def run_if_interactive(self):
        """
        Run the job as Python library and store the result in the HDF5 File.

        Returns:
            int: job ID
        """
        from pyiron.testing.executable import ExampleExecutable

        self._interactive_library = True
        self.status.running = True
        alat, count, energy = ExampleExecutable().run_lib(self.input)
        self._interactive_cache["alat"].append(alat)
        self._interactive_cache["count"].append(count)
        self._interactive_cache["energy"].append(energy)

    def interactive_close(self):
        self._interactive_library = False
        self.to_hdf()
        with self.project_hdf5.open("output") as h5:
            h5["generic/energy"] = np.array(self._interactive_cache["energy"])
            h5["generic/volume"] = np.array(self._interactive_cache["alat"])
            h5["generic/alat"] = np.array(self._interactive_cache["alat"])
            h5["generic/count"] = np.array(self._interactive_cache["count"])
            h5["generic/energy_tot"] = np.array(self._interactive_cache["energy"])
        self.project.db.item_update(self._runtime(), self._job_id)
        self.status.finished = True


class ExampleInput(GenericParameters):
    """
    Input class for the ExampleJob based on the GenericParameters class.

    Args:
        input_file_name (str): Name of the input file - optional
    """

    def __init__(self, input_file_name=None):
        super(ExampleInput, self).__init__(
            input_file_name=input_file_name, table_name="input_inp", comment_char="#"
        )

    def load_default(self):
        """
        Loading the default settings for the input file.
        """
        input_str = """\
alat  3.2     # lattice constant (would be in a more realistic example in the structure file)
alpha 0.1     # noise amplitude
a_0   3       # equilibrium lattice constant
a_1   0
a_2   1.0     # 2nd order in energy (corresponds to bulk modulus)
a_3   0.0     # 3rd order
a_4   0.0     # 4th order
count 10      # number of calls (dummy)
write_restart True
read_restart False
"""
        self.load_string(input_str)


class AtomisticExampleJob(ExampleJob, GenericInteractive):
    """
    ExampleJob generating a list of random numbers to simulate energy fluctuations.

    Args:
        project (ProjectHDFio): ProjectHDFio instance which points to the HDF5 file the job is stored in
        job_name (str): name of the job, which has to be unique within the project

    Attributes:

        .. attribute:: job_name

            name of the job, which has to be unique within the project

        .. attribute:: status

            execution status of the job, can be one of the following [initialized, appended, created, submitted, running,
                                                                      aborted, collect, suspended, refresh, busy, finished]

        .. attribute:: job_id

            unique id to identify the job in the pyiron database

        .. attribute:: parent_id

            job id of the predecessor job - the job which was executed before the current one in the current job series

        .. attribute:: master_id

            job id of the master job - a meta job which groups a series of jobs, which are executed either in parallel or in
            serial.

        .. attribute:: child_ids

            list of child job ids - only meta jobs have child jobs - jobs which list the meta job as their master

        .. attribute:: project

            Project instance the jobs is located in

        .. attribute:: project_hdf5

            ProjectHDFio instance which points to the HDF5 file the job is stored in

        .. attribute:: job_info_str

            short string to describe the job by it is job_name and job ID - mainly used for logging

        .. attribute:: working_directory

            working directory of the job is executed in - outside the HDF5 file

        .. attribute:: path

            path to the job as a combination of absolute file system path and path within the HDF5 file.

        .. attribute:: version

            Version of the hamiltonian, which is also the version of the executable unless a custom executable is used.

        .. attribute:: executable

            Executable used to run the job - usually the path to an external executable.

        .. attribute:: library_activated

            For job types which offer a Python library pyiron can use the python library instead of an external executable.

        .. attribute:: server

            Server object to handle the execution environment for the job.

        .. attribute:: queue_id

            the ID returned from the queuing system - it is most likely not the same as the job ID.

        .. attribute:: logger

            logger object to monitor the external execution and internal pyiron warnings.

        .. attribute:: restart_file_list

            list of files which are used to restart the calculation from these files.

        .. attribute:: job_type

            Job type object with all the available job types: ['ExampleJob', 'SerialMaster', 'ParallelMaster', 'ScriptJob',
                                                               'ListMaster']
    """

    def __init__(self, project, job_name):
        super(AtomisticExampleJob, self).__init__(project, job_name)
        self.__version__ = "0.3"
        self.__name__ = "AtomisticExampleJob"
        self.input = ExampleInput()
        self.executable = "python -m pyiron.testing.executable"
        self.interactive_cache = {
            "cells": [],
            "energy_pot": [],
            "energy_tot": [],
            "forces": [],
            "positions": [],
            "pressures": [],
            "stress": [],
            "steps": [],
            "temperature": [],
            "indices": [],
            "computation_time": [],
            "unwrapped_positions": [],
            "atom_spin_constraints": [],
            "atom_spins": [],
            "magnetic_forces": [],
            "volume": [],
        }

    @property
    def structure(self):
        """

        Returns:

        """
        return self._structure

    def get_structure(self, iteration_step=-1, wrap_atoms=True):
        structure = super(AtomisticExampleJob, self).get_structure(
            iteration_step=iteration_step, wrap_atoms=wrap_atoms
        )
        if structure is None:
            return self.structure
        return structure

    @structure.setter
    def structure(self, structure):
        """

        Args:
            structure:

        Returns:

        """
        self._structure = structure
        if structure is not None:
            self.input["alat"] = self._structure.cell[0, 0]
            # print("set alat: {}".format(self.input["alat"]))

    def set_input_to_read_only(self):
        """
        This function enforces read-only mode for the input classes, but it has to be implement in the individual
        classes.
        """
        super(AtomisticExampleJob, self).set_input_to_read_only()
        self.input.read_only = True

    def to_hdf(self, hdf=None, group_name=None):
        """
        Store the ExampleJob object in the HDF5 File

        Args:
            hdf (ProjectHDFio): HDF5 group object - optional
            group_name (str): HDF5 subgroup name - optional
        """
        super(AtomisticExampleJob, self).to_hdf(hdf=hdf, group_name=group_name)
        self._structure_to_hdf()

    def from_hdf(self, hdf=None, group_name=None):
        """
        Restore the ExampleJob object in the HDF5 File

        Args:
            hdf (ProjectHDFio): HDF5 group object - optional
            group_name (str): HDF5 subgroup name - optional
        """
        super(AtomisticExampleJob, self).from_hdf(hdf=hdf, group_name=group_name)
        self._structure_from_hdf()

    def run_if_interactive(self):
        """
        Run the job as Python library and store the result in the HDF5 File.

        Returns:
            int: job ID
        """
        super(AtomisticExampleJob, self).run_if_interactive()
        self.interactive_cache["cells"].append(self._structure.cell)
        self.interactive_cache["energy_pot"].append(
            self._interactive_cache["energy"][-1][-1]
        )
        self.interactive_cache["energy_tot"].append(
            self._interactive_cache["energy"][-1][-1]
        )
        self.interactive_cache["forces"].append(
            np.random.random((len(self._structure), 3))
        )
        self.interactive_cache["positions"].append(self._structure.positions)
        self.interactive_cache["pressures"].append(np.random.random((3, 3)))
        self.interactive_cache["stress"].append(
            np.random.random((len(self._structure), 3, 3))
        )
        self.interactive_cache["steps"].append(len(self.interactive_cache["steps"]))
        self.interactive_cache["temperature"].append(np.random.random())
        self.interactive_cache["indices"].append(self._structure.indices)
        self.interactive_cache["computation_time"].append(np.random.random())
        self.interactive_cache["unwrapped_positions"].append(self._structure.positions)
        self.interactive_cache["volume"].append(self._structure.get_volume())



# coding: utf-8
# Copyright (c) Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department
# Distributed under the terms of "New BSD License", see the LICENSE file.

from __future__ import print_function
from collections import OrderedDict
import inspect
from pyiron.atomistics.structure.atoms import Atoms
from pyiron.base.master.parallel import ParallelMaster, JobGenerator
from pyiron.atomistics.job.atomistic import AtomisticGenericJob
from pyiron.base.master.generic import get_function_from_string

__author__ = "Jan Janssen"
__copyright__ = (
    "Copyright 2019, Max-Planck-Institut für Eisenforschung GmbH - "
    "Computational Materials Design (CM) Department"
)
__version__ = "1.0"
__maintainer__ = "Jan Janssen"
__email__ = "janssen@mpie.de"
__status__ = "production"
__date__ = "Sep 1, 2017"


class AtomisticParallelMaster(ParallelMaster, AtomisticGenericJob):
    def __init__(self, project, job_name):
        super(AtomisticParallelMaster, self).__init__(project, job_name=job_name)

    @property
    def structure(self):
        if self.ref_job:
            return self._ref_job.structure
        else:
            return None

    @structure.setter
    def structure(self, basis):
        if self.ref_job:
            self._ref_job.structure = basis
        else:
            raise ValueError(
                "A structure can only be set after a reference job has been assinged."
            )

    def get_structure(self, iteration_step=-1):
        if iteration_step == 0:
            return self.structure
        else:
            raise ValueError("iteration_step should be either 0.")


class GenericOutput(OrderedDict):
    def __init__(self):
        super(GenericOutput, self).__init__()


class MapMaster(AtomisticParallelMaster):
    def __init__(self, project, job_name):
        """

        Args:
            project:
            job_name:
        """
        super(MapMaster, self).__init__(project, job_name)
        self.__name__ = "MapMaster"
        self.__version__ = "0.0.1"
        self._job_generator = MapJobGenerator(self)
        self._map_function = None
        self.parameter_list = []

    @property
    def modify_function(self):
        return self._map_function

    @modify_function.setter
    def modify_function(self, funct):
        self._map_function = funct

    def to_hdf(self, hdf=None, group_name=None):
        """
        Store the ParameterMaster in an HDF5 file

        Args:
            hdf (ProjectHDFio): HDF5 group object - optional
            group_name (str): HDF5 subgroup name - optional
        """
        super(MapMaster, self).to_hdf(hdf=hdf, group_name=group_name)
        if len(self.parameter_list) != 0:
            with self.project_hdf5.open("input") as hdf5_input:
                first_element = self.parameter_list[0]
                if isinstance(first_element, Atoms):
                    with hdf5_input.open("structures") as hdf5_input_str:
                        for ind, struct in enumerate(self.parameter_list):
                            struct.to_hdf(
                                hdf=hdf5_input_str, group_name="s_" + str(ind)
                            )
                elif isinstance(first_element, (int, float, str, list)):
                    hdf5_input["parameters_list"] = self.parameter_list
                else:
                    raise TypeError()
                if self._map_function is not None:
                    try:
                        hdf5_input["map_function"] = inspect.getsource(
                            self._map_function
                        )
                    except IOError:
                        hdf5_input["map_function"] = "None"
                else:
                    hdf5_input["map_function"] = "None"

    def from_hdf(self, hdf=None, group_name=None):
        """
        Restore the ParameterMaster from an HDF5 file

        Args:
            hdf (ProjectHDFio): HDF5 group object - optional
            group_name (str): HDF5 subgroup name - optional
        """
        super(MapMaster, self).from_hdf(hdf=hdf, group_name=group_name)
        with self.project_hdf5.open("input") as hdf5_input:
            if "structures" in hdf5_input.list_groups():
                with hdf5_input.open("structures") as hdf5_input_str:
                    self.parameter_list = [
                        Atoms().from_hdf(hdf5_input_str, group_name)
                        for group_name in sorted(hdf5_input_str.list_groups())
                    ]
            else:
                self.parameter_list = hdf5_input["parameters_list"]
            function_str = hdf5_input["map_function"]
            if function_str == "None":
                self._map_function = None
            else:
                self._map_function = get_function_from_string(function_str)

    def collect_output(self):
        pass


class MapJobGenerator(JobGenerator):
    @property
    def parameter_list(self):
        """

        Returns:
            (list)
        """
        return self._job.parameter_list

    def modify_job(self, job, parameter):
        return self._job._map_function(job, parameter)



# coding: utf-8
# Copyright (c) Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department
# Distributed under the terms of "New BSD License", see the LICENSE file.

from __future__ import print_function
from collections import OrderedDict
import hashlib
import numpy as np
import warnings
from pyiron.base.generic.parameters import GenericParameters
import scipy.constants as spc

__author__ = "Joerg Neugebauer, Sudarsan Surendralal, Jan Janssen"
__copyright__ = (
    "Copyright 2019, Max-Planck-Institut für Eisenforschung GmbH - "
    "Computational Materials Design (CM) Department"
)
__version__ = "1.0"
__maintainer__ = "Sudarsan Surendralal"
__email__ = "surendralal@mpie.de"
__status__ = "production"
__date__ = "Sep 1, 2017"


class LammpsControl(GenericParameters):
    def __init__(self, input_file_name=None, **qwargs):
        super(LammpsControl, self).__init__(
            input_file_name=input_file_name, table_name="control_inp", comment_char="#"
        )
        self._init_block_dict()

    @property
    def dataset(self):
        return self._dataset

    def _init_block_dict(self):
        block_dict = OrderedDict()
        block_dict["read_restart"] = "read_restart"
        block_dict["structure"] = (
            "units",
            "dimension",
            "boundary",
            "atom_style",
            "read_data",
            "include",
        )
        block_dict["potential"] = (
            "group",
            "set",
            "pair_style",
            "pair_coeff",
            "bond_style",
            "bond_coeff",
            "angle_style",
            "angle_coeff",
            "kspace_style",
            "neighbor",
        )
        block_dict["compute"] = "compute"
        block_dict["control"] = (
            "fix",
            "min_style",
            "min_modify",
            "neigh_modify",
            "velocity",
            "dump",
            "dump_modify",
            "thermo_style",
            "thermo_modify",
            "thermo",
            "timestep",
            "dielectric",
            "fix_modify",
            "reset_timestep",
        )
        block_dict["run"] = ("run", "minimize")
        block_dict["write_restart"] = "write_restart"
        self._block_dict = block_dict

    def load_default(self, file_content=None):
        if file_content is None:
            file_content = (
                "units               metal\n"
                + "dimension           3\n"
                + "boundary            p p p\n"
                + "atom_style          atomic\n"
                + "read_data           structure.inp\n"
                + "include             potential.inp\n"
                + "fix___ensemble      all nve\n"
                + "variable___dumptime equal 100\n"
                + "dump___1            all custom ${dumptime} dump.out id type xsu ysu zsu fx fy fz\n"
                + 'dump_modify___1     sort id format line "%d %d %20.15g %20.15g %20.15g %20.15g %20.15g %20.15g"\n'
                + "thermo_style        custom step temp pe etotal pxx pxy pxz pyy pyz pzz vol\n"
                + "thermo_modify       format float %20.15g\n"
                + "thermo              100\n"
                + "run                 0\n"
            )
        self.load_string(file_content)

    def calc_minimize(
        self, e_tol=0.0, f_tol=1e-8, max_iter=100000, pressure=None, n_print=100
    ):
        max_evaluations = 10 * max_iter
        if pressure is not None:
            if type(pressure) == float or type(pressure) == int:
                pressure = pressure * np.ones(3)
            str_press = ""
            for press, str_axis in zip(pressure, [" x ", " y ", " z "]):
                if press is not None:
                    str_press += str_axis + str(press * 1.0e4)
            if len(str_press) == 0:
                raise ValueError("Pressure values cannot be three times None")
            elif len(str_press) > 1:
                str_press += " couple none"
            self.set(fix___ensemble=r"all box/relax" + str_press)
        else:
            self.remove_keys(["fix___nve"])
        self.set(
            minimize=str(e_tol)
            + " "
            + str(f_tol)
            + " "
            + str(int(max_iter))
            + " "
            + str(int(max_evaluations))
        )
        self.remove_keys(["run", "velocity"])
        self.modify(variable___dumptime="equal " + str(n_print), thermo=n_print)

    def calc_static(self):
        self.set(run="0")
        self.remove_keys(["minimize", "velocity"])

    def set_initial_velocity(
        self,
        temperature,
        seed=None,
        gaussian=False,
        append_value=False,
        zero_lin_momentum=True,
        zero_rot_momentum=True,
        job_name="",
    ):
        """
        Create initial velocities via velocity all create. More information can be found on LAMMPS website:
        https://lammps.sandia.gov/doc/velocity.html

        Args:
            temperature: (int or float)
            seed: (int) Seed for the initial random number generator
            gaussian: (True/False) Create velocity according to the Gaussian distribution (otherwise uniform)
            append_value: (True/False) Add the velocity values to the current velocities (probably not functional now)
            zero_lin_momentum: (True/False) Cancel the total linear momentum
            zero_rot_momentum: (True/False) Cancel the total angular momentum
            job_name: (str) job name to generate seed
        """

        if seed is None:
            seed = self.generate_seed_from_job(job_name=job_name, seed=1)
        arg = ""
        if gaussian:
            arg = " dist gaussian"
        if append_value:
            arg += " sum yes"
        if not zero_lin_momentum:
            arg += " mom no"
        if not zero_rot_momentum:
            arg += " rot no"
        self.modify(
            velocity="all create " + str(temperature) + " " + str(seed) + arg,
            append_if_not_present=True,
        )

    @staticmethod
    def generate_seed_from_job(job_name="", seed=0):
        """
        Generate a unique seed from the job name.

        Args:
            job_name (str): job_name of the current job to generate the seed
            seed (int): integer to access part of the seed

        Returns:
            int: random seed generated based on the hash
        """
        return int(
            str(int(hashlib.sha256(job_name.encode()).hexdigest(), 16))[
                5 * seed : 5 * seed + 5
            ]
        )

    def calc_md(
        self,
        temperature=None,
        pressure=None,
        n_ionic_steps=1000,
        time_step=1.0,
        n_print=100,
        temperature_damping_timescale=100.0,
        pressure_damping_timescale=1000.0,
        seed=None,
        tloop=None,
        initial_temperature=None,
        langevin=False,
        delta_temp=None,
        delta_press=None,
        job_name="",
    ):
        """
        Set an MD calculation within LAMMPS. Nosé Hoover is used by default.

        Args:
            temperature (None/float): Target temperature. If set to None, an NVE calculation is performed.
                                      It is required when the pressure is set or langevin is set
            pressure (None/float/numpy.ndarray/list): Target pressure. If set to None, an NVE or an NVT calculation is
                performed. A length-3 list or array may be given to specify x-, y- and z-components individually. In
                this case, floats and `None` may be mixed to allow relaxation only in particular directions.
            n_ionic_steps (int): Number of ionic steps
            time_step (float): Step size between two steps. In fs if units==metal
            n_print (int):  Print frequency
            temperature_damping_timescale (float): The time associated with the thermostat adjusting the temperature.
                                                   (In fs. After rescaling to appropriate time units, is equivalent to
                                                   Lammps' `Tdamp`.)
            pressure_damping_timescale (float): The time associated with the barostat adjusting the temperature.
                                                (In fs. After rescaling to appropriate time units, is equivalent to
                                                Lammps' `Pdamp`.)
            seed (int):  Seed for the random number generation (required for the velocity creation)
            tloop:
            initial_temperature (None/float):  Initial temperature according to which the initial velocity field
                                               is created. If None, the initial temperature will be twice the target
                                               temperature (which would go immediately down to the target temperature
                                               as described in equipartition theorem). If 0, the velocity field is not
                                               initialized (in which case  the initial velocity given in structure will
                                               be used). If any other number is given, this value is going to be used
                                               for the initial temperature.
            langevin (bool): (True or False) Activate Langevin dynamics
            delta_temp (float): Thermostat timescale, but in your Lammps time units, whatever those are. (DEPRECATED.)
            delta_press (float): Barostat timescale, but in your Lammps time units, whatever those are. (DEPRECATED.)
            job_name (str): Job name of the job to generate a unique random seed.
        """
        # Conversion factors for transfroming pyiron units to Lammps units
        fs_to_ps = spc.femto / spc.pico
        fs_to_s = spc.femto / 1.0
        GPa_to_bar = spc.giga * 1.0 / spc.bar
        GPa_to_Pa = spc.giga
        GPa_to_barye = spc.giga * 1.0 / (1.0e-6 * spc.bar)  # Lammps is in "barye"
        GPa_to_atm = spc.giga * 1.0 / spc.atm
        lammps_unit_conversions = {
            "metal": {"time": fs_to_ps, "pressure": GPa_to_bar},
            "si": {"time": fs_to_s, "pressure": GPa_to_Pa},
            "cgs": {"time": fs_to_s, "pressure": GPa_to_barye},
            "real": {"time": 1, "pressure": GPa_to_atm},
            "electron": {"time": 1, "pressure": GPa_to_Pa},
        }
        time_units = lammps_unit_conversions[self["units"]]["time"]
        pressure_units = lammps_unit_conversions[self["units"]]["pressure"]
        # No need for temperature conversion; pyiron and all available Lammps units are both in Kelvin
        # (well, except unitless Lennard-Jones units...)
        if self["units"] == "lj":
            raise NotImplementedError

        # Transform time
        if time_step is not None:
            try:
                self["timestep"] = time_step * time_units
            except KeyError:
                raise NotImplementedError()

        # Transform thermostat strength
        if delta_temp is not None:
            warnings.warn(
                "WARNING: `delta_temp` is deprecated, please use `temperature_damping_timescale`."
            )
            temperature_damping_timescale = delta_temp
        else:
            temperature_damping_timescale *= time_units

        # Transform barostat strength
        if delta_press is not None:
            warnings.warn(
                "WARNING: `delta_press` is deprecated, please use `pressure_damping_timescale`."
            )
            pressure_damping_timescale = delta_press
        else:
            pressure_damping_timescale *= time_units

        # Apply initial overheating (default uses the theorem of equipartition of energy between KE and PE)
        if initial_temperature is None and temperature is not None:
            initial_temperature = 2 * temperature

        if seed is None:
            seed = self.generate_seed_from_job(job_name=job_name)

        # Set thermodynamic ensemble
        if pressure is not None:  # NPT
            if not hasattr(pressure, "__len__"):
                pressure = pressure * np.ones(3)
            else:
                pressure = np.array(pressure)

            if sum(pressure != None) == 0:
                raise ValueError("Pressure cannot be three times None")

            if len(pressure) != 3:
                raise ValueError("Pressure must be a float or a 3d vector")

            if temperature is None or temperature == 0.0:
                raise ValueError("Target temperature for fix nvt/npt/nph cannot be 0")

            pressure[pressure != None] *= pressure_units

            pressure_string = ""
            for coord, value in zip(["x", "y", "z"], pressure):
                if value is not None:
                    pressure_string += " {0} {1} {1} {2}".format(
                        coord, str(value), str(pressure_damping_timescale)
                    )

            if langevin:  # NPT(Langevin)
                fix_ensemble_str = "all nph" + pressure_string
                self.modify(
                    fix___langevin="all langevin {0} {1} {2} {3} zero yes".format(
                        str(temperature),
                        str(temperature),
                        str(temperature_damping_timescale),
                        str(seed),
                    ),
                    append_if_not_present=True,
                )
            else:  # NPT(Nose-Hoover)
                fix_ensemble_str = "all npt temp {0} {1} {2}".format(
                    str(temperature),
                    str(temperature),
                    str(temperature_damping_timescale),
                )
                fix_ensemble_str += pressure_string
        elif temperature is not None:  # NVT
            if temperature == 0.0:
                raise ValueError("Target temperature for fix nvt/npt/nph cannot be 0.0")

            if langevin:  # NVT(Langevin)
                fix_ensemble_str = "all nve"
                self.modify(
                    fix___langevin="all langevin {0} {1} {2} {3} zero yes".format(
                        str(temperature),
                        str(temperature),
                        str(temperature_damping_timescale),
                        str(seed),
                    ),
                    append_if_not_present=True,
                )
            else:  # NVT(Nose-Hoover)
                fix_ensemble_str = "all nvt temp {0} {1} {2}".format(
                    str(temperature),
                    str(temperature),
                    str(temperature_damping_timescale),
                )
        else:  # NVE
            if langevin:
                warnings.warn("Temperature not set; Langevin ignored.")
            fix_ensemble_str = "all nve"
            initial_temperature = 0

        if tloop is not None:
            fix_ensemble_str += " tloop " + str(tloop)

        self.remove_keys(["minimize"])
        self.modify(
            fix___ensemble=fix_ensemble_str,
            variable___dumptime=" equal {} ".format(n_print),
            thermo=int(n_print),
            run=int(n_ionic_steps),
            append_if_not_present=True,
        )

        if initial_temperature > 0:
            self.set_initial_velocity(
                temperature=initial_temperature,
                seed=seed,
                gaussian=True,
                job_name=job_name
            )



__version__ = "0.1"
__all__ = []

from pyiron.project import Project
from pyiron.atomistics.structure.atoms import ase_to_pyiron, pyiron_to_ase, Atoms
from pyiron.base.job.script import Notebook
from pyiron.base.settings.install import install_dialog

from ._version import get_versions

__version__ = get_versions()["version"]
del get_versions


def install():
    install_dialog()



# coding: utf-8
# Copyright (c) Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department
# Distributed under the terms of "New BSD License", see the LICENSE file.

import pandas
import time
from pyiron.base.settings.generic import Settings
from pyiron.base.job.jobtype import static_isinstance

"""
Set of functions to interact with the queuing system directly from within pyiron - optimized for the Sun grid engine.
"""

__author__ = "Jan Janssen"
__copyright__ = (
    "Copyright 2019, Max-Planck-Institut für Eisenforschung GmbH - "
    "Computational Materials Design (CM) Department"
)
__version__ = "1.0"
__maintainer__ = "Jan Janssen"
__email__ = "janssen@mpie.de"
__status__ = "production"
__date__ = "Sep 1, 2017"

QUEUE_SCRIPT_PREFIX = "pi_"

s = Settings()


def queue_table(job_ids=[], project_only=True):
    """
    Display the queuing system table as pandas.Dataframe

    Args:
        job_ids (list): check for a specific list of job IDs - empty list by default
        project_only (bool): Query only for jobs within the current project - True by default

    Returns:
        pandas.DataFrame: Output from the queuing system - optimized for the Sun grid engine
    """
    if project_only and not job_ids:
        return []
    if s.queue_adapter is not None:
        df = s.queue_adapter.get_status_of_my_jobs()
        if not project_only:
            return df[
                [
                    True if QUEUE_SCRIPT_PREFIX in job_name else False
                    for job_name in list(df.jobname)
                ]
            ]
        else:
            job_name_lst = [QUEUE_SCRIPT_PREFIX + str(job_id) for job_id in job_ids]
            return df[
                [
                    True if job_name in job_name_lst else False
                    for job_name in list(df.jobname)
                ]
            ]
    else:
        return None


def queue_check_job_is_waiting_or_running(item):
    """
    Check if a job is still listed in the queue system as either waiting or running.

    Args:
        item (int, GenericJob): Provide either the job_ID or the full hamiltonian

    Returns:
        bool: [True/False]
    """
    que_id = _validate_que_request(item)
    if s.queue_adapter is not None:
        return s.queue_adapter.get_status_of_job(process_id=que_id) in [
            "pending",
            "running",
        ]
    else:
        return None


def queue_info_by_job_id(job_id):
    """
    Display the queuing system info of job by qstat | grep  shell command
    as dictionary

    Args:
        requested_id (int): query for a specific job_id

    Returns:
        dict: Dictionary with the output from the queuing system - optimized for the Sun grid engine
    """
    if s.queue_adapter is not None:
        return s.queue_adapter.get_status_of_job(process_id=job_id)
    else:
        return None


def queue_is_empty():
    """
    Check if the queue table is currently empty - no more jobs to wait for.

    Returns:
        bool: True if the table is empty, else False - optimized for the Sun grid engine
    """
    if s.queue_adapter is not None:
        return len(s.queue_adapter.get_status_of_my_jobs()) == 0
    else:
        return True


def queue_delete_job(item):
    """
    Delete a job from the queuing system

    Args:
        item (int, pyiron.base.job.generic.GenericJob): Provide either the job_ID or the full hamiltonian

    Returns:
        str: Output from the queuing system as string - optimized for the Sun grid engine
    """
    que_id = _validate_que_request(item)
    if s.queue_adapter is not None:
        return s.queue_adapter.delete_job(process_id=que_id)
    else:
        return None


def queue_enable_reservation(item):
    """
    Enable a reservation for a particular job within the queuing system

    Args:
        item (int, pyiron.base.job.generic.GenericJob): Provide either the job_ID or the full hamiltonian

    Returns:
        str: Output from the queuing system as string - optimized for the Sun grid engine
    """
    que_id = _validate_que_request(item)
    if s.queue_adapter is not None:
        return s.queue_adapter.enable_reservation(process_id=que_id)
    else:
        return None


def wait_for_job(job, interval_in_s=5, max_iterations=100):
    """
    Sleep until the job is finished but maximum interval_in_s * max_iterations seconds.

    Args:
        job (pyiron.base.job.generic.GenericJob): Job to wait for
        interval_in_s (int): interval when the job status is queried from the database - default 5 sec.
        max_iterations (int): maximum number of iterations - default 100
    """
    if s.queue_adapter is not None and s.queue_adapter.remote_flag and job.server.queue is not None:
        finished = False
        for _ in range(max_iterations):
            if not job.project.queue_check_job_is_waiting_or_running(job):
                job.transfer_from_remote()
                finished = True
                break
            time.sleep(interval_in_s)
        if not finished:
            raise ValueError("Maximum iterations reached, but the job was not finished.")
    else:
        finished = False
        for _ in range(max_iterations):
            job.refresh_job_status()
            if job.status.finished or job.status.aborted or job.status.not_converged:
                finished = True
                break
            time.sleep(interval_in_s)
        if not finished:
            raise ValueError("Maximum iterations reached, but the job was not finished.")


def _validate_que_request(item):
    """
    Internal function to convert the job_ID or hamiltonian to the queuing system ID.

    Args:
        item (int, pyiron.base.job.generic.GenericJob): Provide either the job_ID or the full hamiltonian

    Returns:
        int: queuing system ID
    """

    if isinstance(item, int):
        que_id = item
    elif static_isinstance(item.__class__, "pyiron.base.job.generic.GenericJob"):
        if item.server.queue_id:
            que_id = item.server.queue_id
        else:
            raise ValueError("This job does not have a queue ID.")
    elif static_isinstance(item.__class__, "pyiron.base.job.core.JobCore"):
        if "server" in item.project_hdf5.list_nodes():
            server_hdf_dict = item.project_hdf5["server"]
            if "qid" in server_hdf_dict.keys():
                que_id = server_hdf_dict["qid"]
            else:
                raise ValueError("This job does not have a queue ID.")
        else:
            raise ValueError("This job does not have a queue ID.")
    else:
        raise TypeError(
            "The queue can either query for IDs or for pyiron GenericJobObjects."
        )
    return que_id



"""
Setuptools based setup module
"""
from setuptools import setup, find_packages
import versioneer

setup(
    name='pyiron',
    version=versioneer.get_version(),
    description='pyiron - an integrated development environment (IDE) for computational materials science.',
    long_description='http://pyiron.org',

    url='https://github.com/pyiron/pyiron',
    author='Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department',
    author_email='janssen@mpie.de',
    license='BSD',

    classifiers=['Development Status :: 5 - Production/Stable',
                 'Topic :: Scientific/Engineering :: Physics',
                 'License :: OSI Approved :: BSD License',
                 'Intended Audience :: Science/Research',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.4',
                 'Programming Language :: Python :: 3.5',
                 'Programming Language :: Python :: 3.6',
                 'Programming Language :: Python :: 3.7'],

    keywords='pyiron',
    packages=find_packages(exclude=["*tests*", "*docs*", "*binder*", "*conda*", "*notebooks*", "*.ci_support*"]),
    install_requires=[
        'ase>=3.17',
        'defusedxml>=0.5.0',
        'dill>=0.3.0',
        'future>=0.17.1',
        'h5io>=0.1.1',
        'h5py>=2.9.0',
        'matplotlib>=2.2.4',
        'mendeleev>=0.5.1',
        'numpy>=1.16.4',
        'pandas>=0.24.2',
        'pathlib2>=2.3.4',
        'phonopy>=2.3.2',
        'psutil>=5.6.3',
        'pyfileindex>=0.0.4',
        'pysqa>=0.0.7',
        'scipy>=1.2.1',
        'six>=1.12.0',
        'spglib>=1.14.1',
        'sqlalchemy>=1.3.8',
        'tables>=3.5.1',
        'tqdm>=4.35.0'
    ],
    cmdclass=versioneer.get_cmdclass(),
    )



# coding: utf-8
# Copyright (c) Max-Planck-Institut für Eisenforschung GmbH - Computational Materials Design (CM) Department
# Distributed under the terms of "New BSD License", see the LICENSE file.

import numpy as np
from pyiron.base.settings.generic import Settings
from ovito import ObjectNode
from ovito.data import Bonds, DataCollection
from ovito.modifiers import (
    CreateBondsModifier,
    CommonNeighborAnalysisModifier,
    CentroSymmetryModifier,
    VoronoiAnalysisModifier,
)

__author__ = "Jan Janssen"
__copyright__ = (
    "Copyright 2019, Max-Planck-Institut für Eisenforschung GmbH - "
    "Computational Materials Design (CM) Department"
)
__version__ = "1.0"
__maintainer__ = "Jan Janssen"
__email__ = "janssen@mpie.de"
__status__ = "development"
__date__ = "Sep 1, 2017"

s = Settings()


def analyse_ovito_cna_adaptive(atoms, mode="total"):
    """

    Args:
        atoms:
        mode:

    Returns:

    """
    s.publication_add(publication())
    if not mode in ["total", "numeric", "str"]:
        raise ValueError("Unsupported mode")
    data = DataCollection.create_from_ase_atoms(atoms.copy())
    node = ObjectNode()
    node.source = data
    node.modifiers.append(
        CommonNeighborAnalysisModifier(
            mode=CommonNeighborAnalysisModifier.Mode.AdaptiveCutoff
        )
    )
    output = node.compute()
    if mode == "total":
        return output.attributes
    else:
        atoms_output = output.to_ase_atoms()
        if mode == "numeric":
            return atoms_output.get_array("Structure Type")
        elif mode == "str":
            cna_property = output.particle_properties.structure_type
            return np.array(
                [
                    cna_property.get_type_by_id(cnatype).name
                    for cnatype in atoms_output.get_array("Structure Type")
                ]
            )


def analyse_ovito_centro_symmetry(atoms, num_neighbors=12):
    """

    Args:
        atoms:
        num_neighbors:

    Returns:

    """
    s.publication_add(publication())
    data = DataCollection.create_from_ase_atoms(atoms.copy())
    node = ObjectNode()
    node.source = data
    node.modifiers.append(CentroSymmetryModifier(num_neighbors=num_neighbors))
    output = node.compute()
    return output.particle_properties["Centrosymmetry"].array


def analyse_ovito_voronoi_volume(atoms):
    """

    Args:
        atoms:

    Returns:

    """
    s.publication_add(publication())
    data = DataCollection.create_from_ase_atoms(atoms.copy())
    node = ObjectNode()
    node.source = data
    node.modifiers.append(VoronoiAnalysisModifier())
    output = node.compute()
    return output.particle_properties["Atomic Volume"].array


def publication():
    return {
        "ovito": {
            "Stukowski_2009": {
                "doi": "10.1088/0965-0393/18/1/015012",
                "url": "https://doi.org/10.1088%2F0965-0393%2F18%2F1%2F015012",
                "year": "2009",
                "month": "dec",
                "publisher": "{IOP} Publishing",
                "volume": "18",
                "number": "1",
                "pages": "015012",
                "author": ["Alexander Stukowski"],
                "title": "Visualization and analysis of atomistic simulation data with "
                "{OVITO}{\textendash}the Open Visualization Tool",
                "journal": "Modelling and Simulation in Materials Science and Engineering",
                "abstract": "The Open Visualization Tool (OVITO) is a new 3D visualization "
                "software designed for post-processing atomistic data obtained "
                "from molecular dynamics or Monte Carlo simulations. Unique "
                "analysis, editing and animations functions are integrated into "
                "its easy-to-use graphical user interface. The software is "
                "written in object-oriented C++, controllable via Python scripts "
                "and easily extendable through a plug-in interface. It is "
                "distributed as open-source software and can be downloaded "
                "from the website http://ovito.sourceforge.net/.",
            }
        }
    }



import os
import sys
import getopt
import h5py
import shutil
from pyiron import Project
from pyiron.base.settings.generic import Settings


s = Settings()


def command_line(argv):
    """
    Parse the command line arguments.

    Args:
    argv: Command line arguments

    """
    input_path = None
    output_path = None
    try:
        opts, args = getopt.getopt(
            argv, "i:o:h", ["input_path=", "output_path=", "help"]
        )
    except getopt.GetoptError:
        print("python -m pyiron.base.job.reloadfile -i <inputpath> -o <outputpath>")
        sys.exit()
    else:
        for opt, arg in opts:
            if opt in ("-i", "--input_path"):
                input_path = os.path.abspath(arg)
            if opt in ("-o", "--output_path"):
                output_path = os.path.abspath(arg)
        with h5py.File(input_path, "r") as f:
            job_name = list(f.keys())[0]
        project_path = os.path.join(os.path.abspath("."), job_name + ".h5")
        shutil.copy(input_path, project_path)

        file = os.path.basename(project_path)
        job_name = os.path.splitext(file)[0]

        db_project_path = s.top_path(project_path)
        project = os.path.dirname(project_path)
        db_project = (project + "/").replace(db_project_path, "")
        job_reload = Project(project).load_from_jobpath(
            job_id=None,
            db_entry={
                "id": 1000,
                "status": "",
                "chemicalformula": "",
                "job": job_name,
                "subjob": "/" + job_name,
                "projectpath": db_project_path,
                "project": db_project,
                "hamilton": "",
                "hamversion": "",
                "parentid": None,
                "masterid": None,
            },
            convert_to_object=True,
        )
        job_reload.status.initialized = True
        job_reload.server.run_mode.modal = True
        job_reload.run()
        shutil.copy(project_path, output_path)
        sys.exit()


if __name__ == "__main__":
    command_line(sys.argv[1:])
